@@ -1,7 +1,7 @@
 /*
  * Copyright 2003-2006 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/pax-utils/scanelf.c,v 1.121 2006/02/05 17:10:52 solar Exp $
+ * $Header: /var/cvsroot/gentoo-projects/pax-utils/scanelf.c,v 1.129 2006/02/18 15:51:11 solar Exp $
  *
  * Copyright 2003-2006 Ned Ludd        - <solar@gentoo.org>
  * Copyright 2004-2006 Mike Frysinger  - <vapier@gentoo.org>
@@ -9,7 +9,7 @@
 
 #include "paxinc.h"
 
-static const char *rcsid = "$Id: scanelf.c,v 1.121 2006/02/05 17:10:52 solar Exp $";
+static const char *rcsid = "$Id: scanelf.c,v 1.129 2006/02/18 15:51:11 solar Exp $";
 #define argv0 "scanelf"
 
 #define IS_MODIFIER(c) (c == '%' || c == '#')
@@ -56,6 +56,7 @@ static char be_verbose = 0;
 static char be_wewy_wewy_quiet = 0;
 static char *find_sym = NULL, *versioned_symname = NULL;
 static char *find_lib = NULL;
+static char *find_section = NULL;
 static char *out_format = NULL;
 static char *search_path = NULL;
 static char fix_elf = 0;
@@ -65,6 +66,7 @@ static char use_ldcache = 0;
 int match_bits = 0;
 caddr_t ldcache = 0;
 size_t ldcache_size = 0;
+unsigned long setpax = 0UL;
 
 /* sub-funcs for scanelf_file() */
 static void scanelf_file_get_symtabs(elfobj *elf, void **sym, void **tab)
@@ -93,6 +95,7 @@ static void scanelf_file_get_symtabs(elfobj *elf, void **sym, void **tab)
 	GET_SYMTABS(32)
 	GET_SYMTABS(64)
 }
+
 static char *scanelf_file_pax(elfobj *elf, char *found_pax)
 {
 	static char ret[7];
@@ -111,7 +114,11 @@ static char *scanelf_file_pax(elfobj *elf, char *found_pax)
 	for (i = 0; i < EGET(ehdr->e_phnum); i++) { \
 		if (EGET(phdr[i].p_type) != PT_PAX_FLAGS) \
 			continue; \
-		if (be_quiet && (EGET(phdr[i].p_flags) == 10240)) \
+		if (fix_elf && setpax) { \
+			/* set the paxctl flags */ \
+			ESET(phdr[i].p_flags, setpax); \
+		} \
+		if (be_quiet && (EGET(phdr[i].p_flags) == (PF_NOEMUTRAMP | PF_NORANDEXEC))) \
 			continue; \
 		memcpy(ret, pax_short_pf_flags(EGET(phdr[i].p_flags)), 6); \
 		*found_pax = 1; \
@@ -123,6 +130,18 @@ static char *scanelf_file_pax(elfobj *elf, char *found_pax)
 	SHOW_PAX(64)
 	}
 
+
+	if (fix_elf && setpax) {
+		/* set the chpax settings */
+		if (elf->elf_class == ELFCLASS32) {
+			if (EHDR32(elf->ehdr)->e_type == ET_DYN || EHDR32(elf->ehdr)->e_type == ET_EXEC)
+				ESET(EHDR32(elf->ehdr)->e_ident[EI_PAX],  pax_pf2hf_flags(setpax));
+		} else {
+			if (EHDR64(elf->ehdr)->e_type == ET_DYN || EHDR64(elf->ehdr)->e_type == ET_EXEC)
+				ESET(EHDR64(elf->ehdr)->e_ident[EI_PAX],  pax_pf2hf_flags(setpax));
+		}
+	}
+
 	/* fall back to EI_PAX if no PT_PAX was found */
 	if (!*ret) {
 		static char *paxflags;
@@ -872,6 +891,33 @@ break_out:
 }
 
 
+static char *scanelf_file_sections(elfobj *elf, char *found_section)
+{
+	if (!find_section)
+		 return NULL;
+
+#define FIND_SECTION(B) \
+	if (elf->elf_class == ELFCLASS ## B) { \
+		Elf ## B ## _Shdr *section; \
+		section = SHDR ## B (elf_findsecbyname(elf, find_section)); \
+		if (section != NULL) \
+			*found_section = 1; \
+	}
+	FIND_SECTION(32)
+	FIND_SECTION(64)
+
+
+	if (be_wewy_wewy_quiet) return NULL;
+
+	if (*found_section)
+		return find_section;
+
+	if (be_quiet)
+		return NULL;
+	else
+		return (char *)" - ";
+}
+
 /* scan an elf file and show all the fun stuff */
 #define prints(str) write(fileno(stdout), str, strlen(str))
 static int scanelf_elfobj(elfobj *elf)
@@ -879,13 +925,13 @@ static int scanelf_elfobj(elfobj *elf)
 	unsigned long i;
 	char found_pax, found_phdr, found_relro, found_load, found_textrel, 
 	     found_rpath, found_needed, found_interp, found_bind, found_soname, 
-	     found_sym, found_lib, found_file, found_textrels;
+	     found_sym, found_lib, found_file, found_textrels, found_section;
 	static char *out_buffer = NULL;
 	static size_t out_len;
 
 	found_pax = found_phdr = found_relro = found_load = found_textrel = \
 	found_rpath = found_needed = found_interp = found_bind = found_soname = \
-	found_sym = found_lib = found_file = found_textrels = 0;
+	found_sym = found_lib = found_file = found_textrels = found_section = 0;
 
 	if (be_verbose > 2)
 		printf("%s: scanning file {%s,%s}\n", elf->filename,
@@ -924,6 +970,7 @@ static int scanelf_elfobj(elfobj *elf)
 			case 's': prints("SYM "); break;
 			case 'N': prints("LIB "); break;
 			case 'T': prints("TEXTRELS "); break;
+			case 'k': prints("SECTION "); break;
 			default: warnf("'%c' has no title ?", out_format[i]);
 			}
 		}
@@ -938,10 +985,6 @@ static int scanelf_elfobj(elfobj *elf)
 		const char *out;
 		const char *tmp;
 
-		/* make sure we trim leading spaces in quiet mode */
-		if (be_quiet && *out_buffer == ' ' && !out_buffer[1])
-			*out_buffer = '\0';
-
 		if (!IS_MODIFIER(out_format[i])) {
 			xchrcat(&out_buffer, out_format[i], &out_len);
 			continue;
@@ -991,6 +1034,7 @@ static int scanelf_elfobj(elfobj *elf)
 		case 'b': out = scanelf_file_bind(elf, &found_bind); break;
 		case 'S': out = scanelf_file_soname(elf, &found_soname); break;
 		case 's': out = scanelf_file_sym(elf, &found_sym); break;
+		case 'k': out = scanelf_file_sections(elf, &found_section); break;
 		default: warnf("'%c' has no scan code?", out_format[i]);
 		}
 		if (out) {
@@ -1005,7 +1049,7 @@ static int scanelf_elfobj(elfobj *elf)
 #define FOUND_SOMETHING() \
 	(found_pax || found_phdr || found_relro || found_load || found_textrel || \
 	 found_rpath || found_needed || found_interp || found_bind || \
-	 found_soname || found_sym || found_lib || found_textrels)
+	 found_soname || found_sym || found_lib || found_textrels || found_section )
 
 	if (!found_file && (!be_quiet || (be_quiet && FOUND_SOMETHING()))) {
 		xchrcat(&out_buffer, ' ', &out_len);
@@ -1048,11 +1092,11 @@ static int scanelf_elf(const char *filename, int fd, size_t len)
 			char *p;
 			while((p = strrchr(sbuf, ',')) != NULL) {
 				*p = 0;
-				if (atoi(p+1) == get_etype(elf))
+				if (etype_lookup(p+1) == get_etype(elf))
 					goto label_ret;
 			}
 		}
-		if (atoi(sbuf) != get_etype(elf))
+		if (etype_lookup(sbuf) != get_etype(elf))
 			goto label_done;
 	}
 
@@ -1193,24 +1237,56 @@ static int scanelf_from_file(char *filename)
 	return 0;
 }
 
-static void load_ld_so_conf()
+static int load_ld_so_conf(int i, const char *fname)
 {
 	FILE *fp = NULL;
 	char *p;
 	char path[__PAX_UTILS_PATH_MAX];
-	int i = 0;
 
-	if ((fp = fopen("/etc/ld.so.conf", "r")) == NULL)
-		return;
+	if (i + 1 == sizeof(ldpaths) / sizeof(*ldpaths))
+		return i;
+
+	if ((fp = fopen(fname, "r")) == NULL)
+		return i;
 
 	while ((fgets(path, __PAX_UTILS_PATH_MAX, fp)) != NULL) {
-		if (*path != '/')
-			continue;
-
 		if ((p = strrchr(path, '\r')) != NULL)
 			*p = 0;
 		if ((p = strchr(path, '\n')) != NULL)
 			*p = 0;
+#ifdef HAVE_GLOB
+		// recursive includes of the same file will make this segfault.
+		if ((memcmp(path, "include", 7) == 0) && isblank(path[7])) {
+			glob64_t gl;
+			size_t x;
+			char gpath[__PAX_UTILS_PATH_MAX];
+
+			memset(gpath, 0, sizeof(gpath));
+
+			if (path[8] != '/')
+				snprintf(gpath, sizeof(gpath), "/etc/%s", &path[8]);
+			else
+				strncpy(gpath, &path[8], sizeof(gpath));
+
+			if ((glob64(gpath, 0, NULL, &gl)) == 0) {
+				for (x = 0; x < gl.gl_pathc; ++x) {
+					/* try to avoid direct loops */
+					if (strcmp(gl.gl_pathv[x], fname) == 0)
+						continue;
+					i = load_ld_so_conf(i, gl.gl_pathv[x]);
+					if (i + 1 >= sizeof(ldpaths) / sizeof(*ldpaths)) {
+						globfree64(&gl);
+						return i;
+					}
+				}
+				globfree64 (&gl);
+				continue;
+			} else
+				abort();
+		}
+#endif
+		if (*path != '/')
+			continue;
 
 		ldpaths[i++] = xstrdup(path);
 
@@ -1220,6 +1296,7 @@ static void load_ld_so_conf()
 	ldpaths[i] = NULL;
 
 	fclose(fp);
+	return i;
 }
 
 /* scan /etc/ld.so.conf for paths */
@@ -1264,9 +1341,8 @@ static void scanelf_envpath()
 	free(path);
 }
 
-
 /* usage / invocation handling functions */
-#define PARSE_FLAGS "plRmyAXxetrnLibSs:gN:TaqvF:f:o:M:E:BhV"
+#define PARSE_FLAGS "plRmyAXz:xetrnLibSs:k:gN:TaqvF:f:o:E:M:BhV"
 #define a_argument required_argument
 static struct option const long_opts[] = {
 	{"path",      no_argument, NULL, 'p'},
@@ -1277,6 +1353,7 @@ static struct option const long_opts[] = {
 	{"archives",  no_argument, NULL, 'A'},
 	{"ldcache",   no_argument, NULL, 'L'},
 	{"fix",       no_argument, NULL, 'X'},
+	{"setpax",     a_argument, NULL, 'z'},
 	{"pax",       no_argument, NULL, 'x'},
 	{"header",    no_argument, NULL, 'e'},
 	{"textrel",   no_argument, NULL, 't'},
@@ -1286,6 +1363,7 @@ static struct option const long_opts[] = {
 	{"bind",      no_argument, NULL, 'b'},
 	{"soname",    no_argument, NULL, 'S'},
 	{"symbol",     a_argument, NULL, 's'},
+	{"section",    a_argument, NULL, 'k'},
 	{"lib",        a_argument, NULL, 'N'},
 	{"gmatch",    no_argument, NULL, 'g'},
 	{"textrels",  no_argument, NULL, 'T'},
@@ -1311,7 +1389,8 @@ static const char *opts_help[] = {
 	"Don't scan symlinks",
 	"Scan archives (.a files)",
 	"Utilize ld.so.cache information (use with -r/-n)",
-	"Try and 'fix' bad things (use with -r/-e)\n",
+	"Try and 'fix' bad things (use with -r/-e)",
+	"Sets EI_PAX/PT_PAX_FLAGS to <arg> (use with -Xx)\n",
 	"Print PaX markings",
 	"Print GNU_STACK/PT_LOAD markings",
 	"Print TEXTREL information",
@@ -1321,10 +1400,11 @@ static const char *opts_help[] = {
 	"Print BIND information",
 	"Print SONAME information",
 	"Find a specified symbol",
+	"Find a specified section",
 	"Find a specified library",
 	"Use strncmp to match libraries. (use with -N)",
 	"Locate cause of TEXTREL",
-	"Print only ELF files matching numeric constant type",
+	"Print only ELF files matching etype ET_DYN,ET_EXEC ...",
 	"Print only ELF files matching numeric bits",
 	"Print all scanned info (-x -e -t -r -b)\n",
 	"Only output 'bad' things",
@@ -1347,21 +1427,21 @@ static void usage(int status)
 	printf("Options: -[%s]\n", PARSE_FLAGS);
 	for (i = 0; long_opts[i].name; ++i)
 		if (long_opts[i].has_arg == no_argument)
-			printf("  -%c, --%-13s* %s\n", long_opts[i].val, 
+			printf("  -%c, --%-14s* %s\n", long_opts[i].val, 
 			       long_opts[i].name, opts_help[i]);
 		else
-			printf("  -%c, --%-6s <arg> * %s\n", long_opts[i].val,
+			printf("  -%c, --%-7s <arg> * %s\n", long_opts[i].val,
 			       long_opts[i].name, opts_help[i]);
 
 	if (status != EXIT_SUCCESS)
 		exit(status);
-
+	
 	puts("\nThe format modifiers for the -F option are:");
 	puts(" F Filename \tx PaX Flags \te STACK/RELRO");
 	puts(" t TEXTREL  \tr RPATH     \tn NEEDED");
 	puts(" i INTERP   \tb BIND      \ts symbol");
 	puts(" N library  \to Type      \tT TEXTRELs");
-	puts(" S SONAME");
+	puts(" S SONAME   \tk section");
 	puts(" p filename (with search path removed)");
 	puts(" f filename (short name/basename)");
 	puts("Prefix each modifier with '%' (verbose) or '#' (silent)");
@@ -1406,7 +1486,10 @@ static void parseargs(int argc, char *argv[])
 			SET_STDOUT(fp);
 			break;
 		}
-
+		case 'k':
+			if (find_section) warn("You prob don't want to specify -k twice");
+			find_section = optarg;
+			break;
 		case 's': {
 			if (find_sym) warn("You prob don't want to specify -s twice");
 			find_sym = optarg;
@@ -1425,8 +1508,59 @@ static void parseargs(int argc, char *argv[])
 			out_format = optarg;
 			break;
 		}
+		case 'z': {
+			unsigned long flags = (PF_NOEMUTRAMP | PF_NORANDEXEC);
+			size_t x;
 
-		case 'g': gmatch = 1; /* break; any reason we dont breal; here ? */
+#define do_state(option, flag)				\
+			if (islower(option)) {		\
+				flags &= ~PF_##flag;	\
+				flags |= PF_NO##flag;	\
+			} else {			\
+				flags &= ~PF_NO##flag;	\
+				flags |= PF_##flag;	\
+			}
+
+			for (x = 0 ; x < strlen(optarg); x++) {
+				switch(optarg[x]) {
+					case 'p':
+					case 'P':
+						do_state(optarg[x], PAGEEXEC);
+						break;
+					case 's':
+					case 'S':
+						do_state(optarg[x], SEGMEXEC);
+						break;
+					case 'm':
+					case 'M':
+						do_state(optarg[x], MPROTECT);
+						break;
+					case 'e':
+					case 'E':
+						do_state(optarg[x], EMUTRAMP);
+						break;
+					case 'r':
+					case 'R':
+						do_state(optarg[x], RANDMMAP);
+						break;
+					case 'x':
+					case 'X':
+						do_state(optarg[x], RANDEXEC);
+						break;
+					default:
+						break;
+				}
+			}
+			if (!(((flags & PF_PAGEEXEC) && (flags & PF_NOPAGEEXEC)) ||
+				((flags & PF_SEGMEXEC) && (flags & PF_NOSEGMEXEC)) ||
+				((flags & PF_RANDMMAP) && (flags & PF_NORANDMMAP)) ||
+				((flags & PF_RANDEXEC) && (flags & PF_NORANDEXEC)) ||
+				((flags & PF_EMUTRAMP) && (flags & PF_NOEMUTRAMP)) ||
+				((flags & PF_RANDMMAP) && (flags & PF_NORANDMMAP))))
+					setpax = flags;
+			break;
+		}
+		case 'g': gmatch = 1; break;
 		case 'L': use_ldcache = 1; break;
 		case 'y': scan_symlink = 0; break;
 		case 'A': scan_archives = 1; break;
@@ -1472,6 +1606,7 @@ static void parseargs(int argc, char *argv[])
 			case 'F': break;
 			case 'p': break;
 			case 'f': break;
+			case 'k': break;
 			case 's': break;
 			case 'N': break;
 			case 'o': break;
@@ -1505,6 +1640,7 @@ static void parseargs(int argc, char *argv[])
 		if (show_soname)   xstrcat(&out_format, "%S ", &fmt_len);
 		if (show_textrels) xstrcat(&out_format, "%T ", &fmt_len);
 		if (find_sym)      xstrcat(&out_format, "%s ", &fmt_len);
+		if (find_section)  xstrcat(&out_format, "%k ", &fmt_len);
 		if (find_lib)      xstrcat(&out_format, "%N ", &fmt_len);
 		if (!be_quiet)     xstrcat(&out_format, "%F ", &fmt_len);
 	}
@@ -1512,7 +1648,7 @@ static void parseargs(int argc, char *argv[])
 
 	/* now lets actually do the scanning */
 	if (scan_ldpath || use_ldcache)
-		load_ld_so_conf();
+		load_ld_so_conf(0, "/etc/ld.so.conf");
 	if (scan_ldpath) scanelf_ldpath();
 	if (scan_envpath) scanelf_envpath();
 	if (from_file) {
